// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.strategy.feature;

import de.uka.ilkd.key.logic.PosInOccurrence;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.proof.Goal;
import de.uka.ilkd.key.rule.RuleApp;
import de.uka.ilkd.key.strategy.RuleAppCost;
import de.uka.ilkd.key.strategy.termProjection.ProjectionToTerm;
import de.uka.ilkd.key.strategy.termfeature.TermFeature;

/**
 * Feature for invoking a term feature on the instantiation of a schema variable
 */
public class ApplyTFFeature implements Feature {

    private final ProjectionToTerm proj;
    private final TermFeature      termFeature;
    private final RuleAppCost      noInstCost;

    /**
     * @param schemaVar
     *            the schema variable whose instantiation is supposed to be
     *            inspected
     * @param termFeature
     *            the term feature to use
     * @param noInstCost
     *            result if <code>schemaVar</code> is not instantiated
     * @param demandInst
     *            if <code>true</code> then raise an exception if
     *            <code>schemaVar</code> is not instantiated (otherwise:
     *            return <code>noInstCost</code>)
     */
    private ApplyTFFeature(ProjectionToTerm proj, TermFeature termFeature,
                           RuleAppCost noInstCost) {
        this.proj = proj;
        this.termFeature = termFeature;
        this.noInstCost = noInstCost;
    }

    public static Feature create(ProjectionToTerm proj, TermFeature tf,
                                 RuleAppCost noInstCost) {
        return new ApplyTFFeature ( proj, tf, noInstCost );
    }

    public RuleAppCost compute(RuleApp app, PosInOccurrence pos, Goal goal) {
        final Term te = proj.toTerm ( app, pos, goal );
        if ( te == null ) return noInstCost;

        return termFeature.compute ( te );
    }

}
