// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//


package de.uka.ilkd.key.strategy.feature;

import de.uka.ilkd.key.logic.PosInOccurrence;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.ldt.IntegerLDT;
import de.uka.ilkd.key.logic.op.Function;
import de.uka.ilkd.key.proof.Goal;
import de.uka.ilkd.key.rule.TacletApp;
import de.uka.ilkd.key.strategy.termProjection.ProjectionToTerm;


/**
 * Feature that returns zero iff each variable/atom of one monomial is smaller
 * than all variables of a second monomial
 */
public class AtomsSmallerThanFeature extends SmallerThanFeature {

    private final ProjectionToTerm left, right;
    private final Function mul;


    private AtomsSmallerThanFeature(ProjectionToTerm left, ProjectionToTerm right, 
                                    IntegerLDT numbers) {
        this.left = left;
        this.right = right;
        this.mul = numbers.getArithMultiplication();
    }


    public static Feature create(ProjectionToTerm left, ProjectionToTerm right, 
                                 IntegerLDT numbers) {
        return new AtomsSmallerThanFeature ( left, right, numbers );
    }

    protected boolean filter(TacletApp app, PosInOccurrence pos, Goal goal) {
        final AtomCollector m1 = new AtomCollector ();
        m1.collect ( left.toTerm ( app, pos, goal ) );
        final AtomCollector m2 = new AtomCollector ();
        m2.collect ( right.toTerm ( app, pos, goal ) );

        return lessThan ( m1.getResult(), m2.getResult() );
    }

    private class AtomCollector extends Collector {
        protected void collect(Term te) {
            if ( te.op () == mul ) {
                collect ( te.sub ( 0 ) );
                collect ( te.sub ( 1 ) );
            } else {
                addTerm ( te );
            }
        }
    }    
}
